@@ -199,9 +199,48 @@ static char *rcs_id = "$Id: config.c,v 1.9 2005/06/01 15:01:53 roseg Rel roseg $
  *
  */
 
+#ifndef MISS_FACILITYNAMES
 #define SYSLOG_NAMES    1
+#endif
+
 #include    "pound.h"
 
+#ifdef MISS_FACILITYNAMES
+
+/* This is lifted verbatim from the Linux sys/syslog.h */
+
+typedef struct _code {
+	char	*c_name;
+	int	c_val;
+} CODE;
+
+static CODE facilitynames[] = {
+    { "auth", LOG_AUTH },
+    { "authpriv", LOG_AUTHPRIV },
+    { "cron", LOG_CRON },
+    { "daemon", LOG_DAEMON },
+    { "ftp", LOG_FTP },
+    { "kern", LOG_KERN },
+    { "lpr", LOG_LPR },
+    { "mail", LOG_MAIL },
+    { "mark", 0 },                  /* never used! */
+    { "news", LOG_NEWS },
+    { "security", LOG_AUTH },       /* DEPRECATED */
+    { "syslog", LOG_SYSLOG },
+    { "user", LOG_USER },
+    { "uucp", LOG_UUCP },
+    { "local0", LOG_LOCAL0 },
+    { "local1", LOG_LOCAL1 },
+    { "local2", LOG_LOCAL2 },
+    { "local3", LOG_LOCAL3 },
+    { "local4", LOG_LOCAL4 },
+    { "local5", LOG_LOCAL5 },
+    { "local6", LOG_LOCAL6 },
+    { "local7", LOG_LOCAL7 },
+    { NULL, -1 }
+};
+#endif
+
 static char *
 file2str(char *fname)
 {
