@@ -26,10 +26,60 @@
  * EMail: roseg@apsis.ch
  */
 
-static char *rcs_id = "$Id: pound.c,v 1.6 2003/11/30 22:56:26 roseg Rel $";
+static char *rcs_id = "$Id: pound.c,v 1.9 2005/06/01 15:01:53 roseg Rel roseg $";
 
 /*
  * $Log: pound.c,v $
+ * Revision 1.9  2005/06/01 15:01:53  roseg
+ * Enhancements:
+ *   Added the VerifyList configuration flag (CA root certs + CRL)
+ *   CRL checking code
+ *   RewriteRedirect 2 - ignores port value for host matching
+ *   Added -c flag (check-only mode)
+ *   Added -v flag (verbose mode)
+ *   Added -p flag for pid file name
+ *
+ * Bug fixes:
+ *   fixed a potential buffer overflow problem (in checking the Host header)
+ *   added call to SSL_library_init
+ *   added a check for MSIE before forcing SSL shutdown
+ *   X-SSL-Cipher header is added only if HTTPSHeaders is non-zero
+ *   added code for shorter linger on badly closed connections (IE work-around)
+ *   fixed the locking for session checking (mutex_lock/unlock)
+ *
+ * Revision 1.8  2004/11/04 13:37:07  roseg
+ * Changes:
+ * - added support for non-blocking connect(2)
+ * - added support for 414 - Request URI too long
+ * - added RedirectRewrite directive - to prevent redirect changes
+ * - added support for NoHTTPS11 value 2 (for MSIE clients only)
+ * - added support for HTTPSHeaders 3 (no verify)
+ *
+ * Problems fixed:
+ * - fixed bug if multiple listening ports/addresses
+ * - fixed memory leak in SSL
+ * - flush stdout (if used) after each log message
+ * - assumes only 304, 305 and 306 codes to have no content
+ * - fixed problem with delays in 302 without content
+ * - fixed problem with time-outs in HTTPS
+ *
+ * Enhancements:
+ * - improved threads detection code in autoconf
+ * - added supervisor process disable configuration flag
+ * - tweak for the Location rewriting code (only look at current GROUP)
+ * - improved print-out for client certificate information
+ *
+ * Revision 1.7  2004/03/24 06:59:59  roseg
+ * Fixed bug in X-SSL-CIPHER description
+ * Changed README to stx format for consistency
+ * Addedd X-SSL-certificate with full client certificate
+ * Improved the response times on HTTP/0.9 (content without Content-length)
+ * Improved response granularity on above - using unbuffered BIO now
+ * Fixed problem with IE/SSL (SSL_set_shutdown)
+ * Avoid error messages on premature EOF from client
+ * Fixed HeadRemove code so all headers are checked without exception
+ * Improved autoconf detection
+ *
  * Revision 1.6  2003/11/30 22:56:26  roseg
  * Callback for RSA ephemeral keys:
  *     - generated in a separate thread
@@ -178,11 +228,12 @@ int     log_level;          /* logging mode - 0, 1, 2 */
 int     https_headers;      /* add HTTPS-specific headers */
 char    *https_header;      /* HTTPS-specific header to add */
 char    *ssl_CAlst;         /* CA certificate list (path to file) */
+char    *ssl_Verifylst;     /* Verify (path to file) */
 int     ssl_vdepth;         /* max verification depth */
 int     allow_xtd;          /* allow extended HTTP - PUT, DELETE */
 int     allow_dav;          /* allow WebDAV - LOCK, UNLOCK */
 int     no_https_11;        /* disallow HTTP/1.1 clients for SSL connections */
-int     alive_to;           /* allow extended HTTP - PUT, DELETE */
+int     alive_to;           /* check interval for resurrection */
 long    max_req;            /* maximal allowed request size */
 char    **http,             /* HTTP port to listen on */
         **https,            /* HTTPS port to listen on */
@@ -200,6 +251,12 @@ char    **http,             /* HTTP port to listen on */
         *CS_qval,           /* character set of query value */
         *CS_frag;           /* character set of fragment */
 int     check_URL;          /* check URL for correct syntax */
+int     rewrite_redir;      /* rewrite redirection responses */
+int     daemonize;          /* run as daemon */
+int     print_log;          /* print log messages to stdout/stderr */
+char    *pid_name;          /* file to record pid in */
+int     log_facility;       /* log facility to use */
+
 regex_t *head_off;          /* headers to remove */
 int     n_head_off;         /* how many of them */
 
@@ -214,11 +271,13 @@ regex_t HTTP,               /* normal HTTP requests: GET, POST, HEAD */
         RESP_SKIP,          /* responses for which we skip response */
         RESP_IGN,           /* responses for which we ignore content */
         RESP_REDIR,         /* responses for which we rewrite Location */
-        LOCATION;           /* the host we are redirected to */
+        LOCATION,           /* the host we are redirected to */
+        AUTHORIZATION;      /* the Authorisation header */
 
 char    *e500 = "An internal server error occurred. Please try again later.",
         *e501 = "This method may not be used.",
-        *e503 = "The service is not available. Please try again later.";
+        *e503 = "The service is not available. Please try again later.",
+        *e414 = "Request URI is too long.";
 
 /* worker pid */
 static  pid_t               son = 0;
@@ -231,13 +290,14 @@ static pthread_mutex_t  *l_array;
 static void
 l_init(void)
 {
-    int i;
+    int i, n_locks;
 
-    if((l_array = (pthread_mutex_t *)calloc(CRYPTO_num_locks(), sizeof(pthread_mutex_t))) == NULL) {
+    n_locks = CRYPTO_num_locks();
+    if((l_array = (pthread_mutex_t *)calloc(n_locks, sizeof(pthread_mutex_t))) == NULL) {
         logmsg(LOG_ERR, "lock init: out of memory - aborted...");
         exit(1);
     }
-    for(i = 0; i < CRYPTO_num_locks(); i++)
+    for(i = 0; i < n_locks; i++)
         pthread_mutex_init(&l_array[i], NULL);
     return;
 }
@@ -282,7 +342,7 @@ addr_in_use(struct sockaddr_in *addr)
         logmsg(LOG_ERR, "check socket create: %s - aborted", strerror(errno));
         return 1;
     }
-    res = (connect(sock, (struct sockaddr *)addr, (socklen_t)sizeof(*addr)) == 0);
+    res = (connect_nb(sock, (struct sockaddr *)addr, (socklen_t)sizeof(*addr)) == 0);
     close(sock);
     return res;
 }
@@ -293,6 +353,9 @@ addr_in_use(struct sockaddr_in *addr)
 static int
 verify_cert(int pre_ok, X509_STORE_CTX *ctx)
 {
+    if(https_headers > 2)
+        return 1;
+
     if(!pre_ok)
         /* we already had an error */
         return 0;
@@ -318,23 +381,21 @@ main(int argc, char **argv)
 {
     pthread_t           thr;
     pthread_attr_t      attr;
-    int                 *http_sock, *https_sock, clnt_length, i, n, max_fd, clnt;
+    int                 *http_sock, *https_sock, clnt_length, i, n, n_polls, clnt, host_length;
     struct sockaddr_in  host_addr, clnt_addr;
     struct hostent      *host;
-    fd_set              socks;
+    struct pollfd       *polls;
     uid_t               user_id;
     gid_t               group_id;
     FILE                *fpid;
-    char                fpid_name[32];
     regex_t             LISTEN_ADDR;
     regmatch_t          matches[3];
     SSL_CTX             **ctx;
+    X509_STORE*         store;
 
+    print_log = 0;
 #ifndef  NO_SYSLOG
-#ifndef FACILITY
-#define FACILITY    LOG_DAEMON
-#endif
-    openlog("pound", LOG_CONS, FACILITY);
+    openlog("pound", LOG_CONS, LOG_DAEMON);
 #endif
     logmsg(LOG_NOTICE, "starting...");
 
@@ -345,6 +406,14 @@ main(int argc, char **argv)
 
     config_parse(argc, argv);
 
+    /* SSL stuff */
+    SSL_load_error_strings();
+    SSL_library_init();
+    OpenSSL_add_all_algorithms();
+    l_init();
+    CRYPTO_set_id_callback(l_id);
+    CRYPTO_set_locking_callback(l_lock);
+
 #if HAVE_OPENSSL_ENGINE_H
     /* select SSL engine */
     if (ssl_engine != NULL) {
@@ -382,20 +451,23 @@ main(int argc, char **argv)
 #else
     || regcomp(&WEBDAV, "^(LOCK|UNLOCK) ([^ ]+) HTTP/1.[01]$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
 #endif
+#ifdef UNSAFE
+    || regcomp(&HEADER, "^([A-Za-z0-9_.!#%&'`^~$*+|-]*):[ \t]*(.*)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+#else
     || regcomp(&HEADER, "^([A-Za-z][A-Za-z0-9_-]*):[ \t]*(.*)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+#endif
     || regcomp(&CHUNK_HEAD, "^([0-9a-f]+).*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&LISTEN_ADDR, "^([^,]+),([1-9][0-9]*)$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&RESP_SKIP, "^HTTP/1.1 100.*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
-    || regcomp(&RESP_IGN, "^HTTP/1.[01] (10[1-9]|1[1-9][0-9]|204|304).*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&RESP_IGN, "^HTTP/1.[01] (10[1-9]|1[1-9][0-9]|204|30[456]).*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&RESP_REDIR, "^HTTP/1.[01] 30[1237].*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&LOCATION, "(http|https)://([^/]+)/(.*)", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&AUTHORIZATION, "Authorization:[ \t]*Basic[ \t]*([^ \t]*)[ \t]*", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     ) {
         logmsg(LOG_ERR, "bad Regex - aborted");
         exit(1);
     }
 
-    max_fd = 0;
-
     /* get HTTP address and port */
     if(http[0]) {
         for(i = 0; http[i]; i++)
@@ -420,7 +492,7 @@ main(int argc, char **argv)
                  */
                 host_addr.sin_addr.s_addr = INADDR_ANY;
             } else {
-                if((host = gethostbyname(http[i])) == NULL) {
+                if((host = gethostbyname(http[i])) == NULL || host->h_addr_list[0] == NULL) {
                     logmsg(LOG_ERR, "Unknown HTTP host %s", http[i]);
                     exit(1);
                 }
@@ -440,8 +512,6 @@ main(int argc, char **argv)
                     logmsg(LOG_ERR, "HTTP socket create: %s - aborted", strerror(errno));
                     exit(1);
                 }
-                if(http_sock[i] > max_fd)
-                    max_fd = http_sock[i];
                 opt = 1;
                 setsockopt(http_sock[i], SOL_SOCKET, SO_REUSEADDR, (void *)&opt, sizeof(opt));
                 if(bind(http_sock[i], (struct sockaddr *)&host_addr, (socklen_t)sizeof(host_addr)) < 0) {
@@ -453,15 +523,10 @@ main(int argc, char **argv)
         }
     }
 
-    /* SSL stuff */
-    SSL_load_error_strings();
-    OpenSSL_add_all_algorithms();
-    l_init();
-    CRYPTO_set_id_callback(l_id);
-    CRYPTO_set_locking_callback(l_lock);
-
     /* get HTTPS address and port */
     if(https[0]) {
+        init_RSAgen();
+
         for(i = 0; https[i]; i++)
             ;
         if((https_sock = (int *)malloc(sizeof(int) * i)) == NULL) {
@@ -488,7 +553,7 @@ main(int argc, char **argv)
                  */
                 host_addr.sin_addr.s_addr = INADDR_ANY;
             } else {
-                if((host = gethostbyname(https[i])) == NULL) {
+                if((host = gethostbyname(https[i])) == NULL || host->h_addr_list[0] == NULL) {
                     logmsg(LOG_ERR, "Unknown HTTPS host %s", https[i]);
                     exit(1);
                 }
@@ -501,15 +566,14 @@ main(int argc, char **argv)
                 logmsg(LOG_WARNING, "%s:%s already in use - skipped", https[i], https[i] + matches[2].rm_so);
                 https_sock[i] = -1;
             } else {
-                int opt;
+                int     opt;
+                char    sess_id[33];
 
                 /* prepare the socket */
                 if((https_sock[i] = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
                     logmsg(LOG_ERR, "HTTPS socket create: %s - aborted", strerror(errno));
                     exit(1);
                 }
-                if(https_sock[i] > max_fd)
-                    max_fd = https_sock[i];
                 opt = 1;
                 setsockopt(https_sock[i], SOL_SOCKET, SO_REUSEADDR, (void *)&opt, sizeof(opt));
                 if(bind(https_sock[i], (struct sockaddr *)&host_addr, (socklen_t)sizeof(host_addr)) < 0) {
@@ -539,18 +603,30 @@ main(int argc, char **argv)
                 }
 
                 /* additional CTX setup */
-                if(https_headers > 1)
-                    SSL_CTX_set_verify(ctx[i], SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT, verify_cert);
-                else if(https_headers == 1)
-                    SSL_CTX_set_verify(ctx[i], SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE, verify_cert);
-                else
+                switch(https_headers) {
+                case 0:
                     SSL_CTX_set_verify(ctx[i], SSL_VERIFY_NONE, verify_cert);
-                SSL_CTX_set_verify_depth(ctx[i], ssl_vdepth);
+                    break;
+                case 1:
+                case 3:
+                    SSL_CTX_set_verify(ctx[i], SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE, verify_cert);
+                    SSL_CTX_set_verify_depth(ctx[i], ssl_vdepth);
+                    break;
+                case 2:
+                    SSL_CTX_set_verify(ctx[i], SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT, verify_cert);
+                    SSL_CTX_set_verify_depth(ctx[i], ssl_vdepth);
+                    break;
+                }
                 SSL_CTX_set_mode(ctx[i], SSL_MODE_AUTO_RETRY);
                 SSL_CTX_set_options(ctx[i], SSL_OP_ALL);
+
+                snprintf(sess_id, 32, "%d-Pound-%d", getpid(), i);
+                SSL_CTX_set_session_id_context(ctx[i], (unsigned char *)sess_id, strlen(sess_id));
+
                 if(ciphers[i])
                     SSL_CTX_set_cipher_list(ctx[i], ciphers[i]);
                 SSL_CTX_set_tmp_rsa_callback(ctx[i], RSA_tmp_callback);
+
                 if(ssl_CAlst != NULL) {
                     STACK_OF(X509_NAME) *cert_names;
 
@@ -559,17 +635,44 @@ main(int argc, char **argv)
                         exit(1);
                     }
                     SSL_CTX_set_client_CA_list(ctx[i], cert_names);
+                }
 
-                    if(SSL_CTX_load_verify_locations(ctx[i], ssl_CAlst, NULL) != 1) {
+                if(ssl_Verifylst != NULL) {
+                    if(SSL_CTX_load_verify_locations(ctx[i], ssl_Verifylst, NULL) != 1) {
                         logmsg(LOG_ERR, "SSL_CTX_load_verify_locations failed - aborted");
                         exit(1);
                     }
                 }
+#if HAVE_X509_STORE_SET_FLAGS
+                /* add the CRL stuff */
+                if((store = SSL_CTX_get_cert_store(ctx[i])) != NULL)
+                    X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK);
+                else
+                    logmsg(LOG_WARNING, "SSL_CTX_get_cert_store failed!");
+#endif
             }
         }
     }
 
-    max_fd++;
+    /* alloc the poll structures */
+    n_polls = 0;
+    for(i = 0; http[i]; i++)
+        if(http_sock[i] >= 0)
+            n_polls++;
+    for(i = 0; https[i]; i++)
+        if(https_sock[i] >= 0)
+            n_polls++;
+    if((polls = (struct pollfd *)calloc(n_polls, sizeof(struct pollfd))) == NULL) {
+        logmsg(LOG_ERR, "Out of memory for poll - aborted");
+        exit(1);
+    }
+    n = 0;
+    for(i = 0; http[i]; i++)
+        if(http_sock[i] >= 0)
+            polls[n++].fd = http_sock[i];
+    for(i = 0; https[i]; i++)
+        if(https_sock[i] >= 0)
+            polls[n++].fd = https_sock[i];
 
     /* set uid if necessary */
     if(user) {
@@ -593,31 +696,36 @@ main(int argc, char **argv)
         group_id = gr->gr_gid;
     }
 
-#ifdef  AEMON
-    /* daemonize - make ourselves a subprocess. */
-    switch (fork()) {
-        case 0:
-#ifndef NO_SYSLOG
-            close(0);
-            close(1);
-            close(2);
-#endif
-            break;
-        case -1:
-            logmsg(LOG_ERR, "fork: %s - aborted", strerror(errno));
-            exit(1);
-        default:
-            exit(0);
-    }
-#endif
+    /* Turn off verbose messages (if necessary) */
+    print_log = 0;
 
-    /* record pid in var/run */
-    sprintf(fpid_name, "/var/run/pound_pid.%d", getpid());
-    if((fpid = fopen(fpid_name, "wt")) != NULL) {
+    if(daemonize) {
+        /* daemonize - make ourselves a subprocess. */
+        switch (fork()) {
+            case 0:
+#ifndef NO_SYSLOG
+                close(0);
+                close(1);
+                close(2);
+#endif
+                break;
+            case -1:
+                logmsg(LOG_ERR, "fork: %s - aborted", strerror(errno));
+                exit(1);
+            default:
+                exit(0);
+        }
+#ifdef  HAVE_SETSID
+        (void) setsid();
+#endif
+    }
+
+    /* record pid in file */
+    if((fpid = fopen(pid_name, "wt")) != NULL) {
         fprintf(fpid, "%d\n", getpid());
         fclose(fpid);
     } else
-        logmsg(LOG_WARNING, "Create \"%s\": %s", fpid_name, strerror(errno));
+        logmsg(LOG_WARNING, "Create \"%s\": %s", pid_name, strerror(errno));
 
     /* chroot if necessary */
     if(root) {
@@ -642,26 +750,28 @@ main(int argc, char **argv)
             exit(1);
         }
 
-    /* split off into monitor and working process */
-    for(;;)
+    /* split off into monitor and working process if necessary */
+    for(;;) {
+#ifdef  UPER
         if((son = fork()) > 0) {
             int status;
 
             while(wait(&status) != son)
                 logmsg(LOG_ERR, "MONITOR: bad wait (%s)", strerror(errno));
             if(WIFEXITED(status))
-                logmsg(LOG_ERR, "MONITOR: worker exited nurmally %d, restarting...", WEXITSTATUS(status));
+                logmsg(LOG_ERR, "MONITOR: worker exited normally %d, restarting...", WEXITSTATUS(status));
             else if(WIFSIGNALED(status))
                 logmsg(LOG_ERR, "MONITOR: worker exited on signal %d, restarting...", WTERMSIG(status));
             else
                 logmsg(LOG_ERR, "MONITOR: worker exited (stopped?) %d, restarting...", status);
         } else if (son == 0) {
+#endif
             /* thread stuff */
             pthread_attr_init(&attr);
             pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 
 #ifdef  NEED_STACK
-            /* set new stack size - necessary for OpenBSD/FreeBSD */
+            /* set new stack size - necessary for OpenBSD/FreeBSD and Linux NPTL */
             if(pthread_attr_setstacksize(&attr, 1 << 18)) {
                 logmsg(LOG_ERR, "can't set stack size - aborted");
                 exit(1);
@@ -680,29 +790,30 @@ main(int argc, char **argv)
                 exit(1);
             }
 
-            /* start the RSA key generator */
-            if(pthread_create(&thr, &attr, thr_RSAgen, NULL)) {
-                logmsg(LOG_ERR, "create thr_RSAgen: %s - aborted", strerror(errno));
-                exit(1);
-            }
+            /* start the RSA stuff */
+            if(https[0])
+                if(pthread_create(&thr, &attr, thr_RSAgen, NULL)) {
+                    logmsg(LOG_ERR, "create thr_RSAgen: %s - aborted", strerror(errno));
+                    exit(1);
+                }
 
             /* pause to make sure the service threads were started */
-            sleep(3);
+            sleep(2);
 
             /* and start working */
             for(;;) {
-                FD_ZERO(&socks);
-                for(i = 0; http[i]; i++)
-                    if(http_sock[i] >= 0)
-                        FD_SET(http_sock[i], &socks);
-                for(i = 0; https[i]; i++)
-                    if(https_sock[i] >= 0)
-                        FD_SET(https_sock[i], &socks);
-                if((i = select(max_fd, &socks, NULL, NULL, NULL)) < 0) {
-                    logmsg(LOG_WARNING, "select: %s", strerror(errno));
+                for(i = 0; i < n_polls; i++) {
+                    polls[i].events = POLLIN | POLLPRI;
+                    polls[i].revents = 0;
+                }
+                if(poll(polls, n_polls, -1) < 0) {
+                    logmsg(LOG_WARNING, "poll: %s", strerror(errno));
                 } else {
+                    n = -1;
                     for(i = 0; http[i]; i++) {
-                        if(http_sock[i] >= 0 && FD_ISSET(http_sock[i], &socks)) {
+                        if(http_sock[i] >= 0)
+                            n++;
+                        if(polls[n].revents & (POLLIN | POLLPRI)) {
                             memset(&clnt_addr, 0, sizeof(clnt_addr));
                             clnt_length = sizeof(clnt_addr);
                             if((clnt = accept(http_sock[i], (struct sockaddr *)&clnt_addr,
@@ -721,9 +832,10 @@ main(int argc, char **argv)
                                 } else {
                                     arg->sock = clnt;
                                     arg->from_host = clnt_addr.sin_addr;
-                                    memset(&arg->to_host, 0, n = sizeof(arg->to_host));
-                                    getsockname(http_sock[i], (struct sockaddr *)&arg->to_host, &n);
-                                    arg->ctx = NULL;
+                                    memset(&arg->to_host, 0, host_length = sizeof(arg->to_host));
+                                    getsockname(http_sock[i], (struct sockaddr *)&arg->to_host,
+                                        (socklen_t *)&host_length);
+                                    arg->ssl = NULL;
                                     if(pthread_create(&thr, &attr, thr_http, (void *)arg)) {
                                         logmsg(LOG_WARNING, "HTTP pthread_create: %s", strerror(errno));
                                         free(arg);
@@ -734,7 +846,9 @@ main(int argc, char **argv)
                         }
                     }
                     for(i = 0; https[i]; i++) {
-                        if(https_sock[i] >= 0 && FD_ISSET(https_sock[i], &socks)) {
+                        if(https_sock[i] >= 0)
+                            n++;
+                        if(polls[n].revents & (POLLIN | POLLPRI)) {
                             memset(&clnt_addr, 0, sizeof(clnt_addr));
                             clnt_length = sizeof(clnt_addr);
                             if((clnt = accept(https_sock[i], (struct sockaddr *)&clnt_addr,
@@ -753,11 +867,17 @@ main(int argc, char **argv)
                                 } else {
                                     arg->sock = clnt;
                                     arg->from_host = clnt_addr.sin_addr;
-                                    memset(&arg->to_host, 0, n = sizeof(arg->to_host));
-                                    getsockname(https_sock[i], (struct sockaddr *)&arg->to_host, &n);
-                                    arg->ctx = ctx[i];
+                                    memset(&arg->to_host, 0, host_length = sizeof(arg->to_host));
+                                    getsockname(https_sock[i], (struct sockaddr *)&arg->to_host,
+                                        (socklen_t *)&host_length);
+                                    if((arg->ssl = SSL_new(ctx[i])) == NULL) {
+                                        logmsg(LOG_WARNING, "HTTPS SSL_new: failed");
+                                        free(arg);
+                                        close(clnt);
+                                    }
                                     if(pthread_create(&thr, &attr, thr_http, (void *)arg)) {
                                         logmsg(LOG_WARNING, "HTTPS pthread_create: %s", strerror(errno));
+                                        SSL_free(arg->ssl);
                                         free(arg);
                                         close(clnt);
                                     }
@@ -767,9 +887,12 @@ main(int argc, char **argv)
                     }
                 }
             }
+#ifdef  UPER
         } else {
             /* failed to spawn son */
             logmsg(LOG_ERR, "Can't fork worker (%s) - aborted", strerror(errno));
             exit(1);
         }
+#endif
+    }
 }
