@@ -27,7 +27,7 @@
  */
 
 /*
- * $Id: pound.h,v 1.6 2003/11/30 22:56:26 roseg Rel $
+ * $Id: pound.h,v 1.9 2005/06/01 15:01:54 roseg Rel roseg $
  * Revision 1.0  2002/10/31 15:21:25  roseg
  * fixed ordering of certificate file
  * removed thread auto clean-up (bug in Linux implementation of libpthread)
@@ -96,6 +96,10 @@
 #error "Pound needs unistd.h"
 #endif
 
+#if HAVE_GETOPT_H
+#include    <getopt.h>
+#endif
+
 #if HAVE_PTHREAD_H
 #include    <pthread.h>
 #else
@@ -147,6 +151,12 @@
 #error "Pound needs netinet/in.h"
 #endif
 
+#if HAVE_NETINET_TCP_H
+#include    <netinet/tcp.h>
+#else
+#error "Pound needs netinet/tcp.h"
+#endif
+
 #if HAVE_ARPA_INET_H
 #include    <arpa/inet.h>
 #else
@@ -159,6 +169,12 @@
 #error "Pound needs netdb.h"
 #endif
 
+#if HAVE_SYS_POLL_H
+#include    <sys/poll.h>
+#else
+#error "Pound needs sys/poll.h"
+#endif
+
 #if HAVE_OPENSSL_SSL_H
 #define OPENSSL_THREAD_DEFINES
 #include    <openssl/ssl.h>
@@ -193,8 +209,10 @@
 
 #if HAVE_SYSLOG_H
 #include    <syslog.h>
-#else
-#error "Pound needs syslog.h"
+#endif
+
+#if HAVE_SYS_SYSLOG_H
+#include    <sys/syslog.h>
 #endif
 
 #if HAVE_SIGNAL_H
@@ -247,8 +265,6 @@
 #include    <varargs.h>
 #endif
 
-extern int errno;
-
 /*
  * Global variables needed by everybody
  */
@@ -259,6 +275,7 @@ extern int  log_level;          /* logging mode - 0, 1, 2 */
 extern int  https_headers;      /* add HTTPS-specific headers */
 extern char *https_header;      /* HTTPS-specific header to add */
 extern char *ssl_CAlst;         /* CA certificate list (path to file) */
+extern char *ssl_Verifylst;     /* Verify (path to file) */
 extern int  ssl_vdepth;         /* max verification depth */
 extern int  allow_xtd;          /* allow extended HTTP - PUT, DELETE */
 extern int  allow_dav;          /* allow WebDAV - LOCK, UNLOCK */
@@ -281,16 +298,29 @@ extern char **http,             /* HTTP port to listen on */
             *CS_qval,           /* character set of query value */
             *CS_frag;           /* character set of fragment */
 extern int  check_URL;          /* check URL for correct syntax */
+extern int  rewrite_redir;      /* rewrite redirection responses */
+extern int  daemonize;          /* run as daemon */
+extern int  print_log;          /* print log messages to stdout/stderr */
+extern char *pid_name;          /* file to record pid in */
+extern int  log_facility;       /* log facility to use */
 
 extern regex_t  *head_off;          /* headers to remove */
 extern int      n_head_off;         /* how many of them */
 
-#define MAXBUF      8192
+#define MAXBUF      2048
 #define MAXHEADERS  128
 #define MAXCHAIN    8
 #define GLOB_SESS   15
-#define N_RSA_KEYS  11
-#define T_RSA_KEYS  300
+
+#ifndef F_CONF
+#define F_CONF  "/usr/local/etc/pound.cfg"
+#endif
+
+#ifndef F_PID
+#define F_PID  "/var/run/pound.pid"
+#endif
+
+#define SERVER_TO   (server_to > 0? server_to: 5)
 
 /* Backend definition */
 typedef struct {
@@ -337,7 +367,7 @@ typedef struct  {
     int                 sock;
     struct in_addr      from_host;
     struct sockaddr_in  to_host;
-    SSL_CTX             *ctx;
+    SSL                 *ssl;
 }   thr_arg;                        /* argument to processing threads: socket, origin */
 
 extern regex_t  HTTP,       /* normal HTTP requests: GET, POST, HEAD */
@@ -349,11 +379,13 @@ extern regex_t  HTTP,       /* normal HTTP requests: GET, POST, HEAD */
                 RESP_SKIP,  /* responses for which we skip response */
                 RESP_IGN,   /* responses for which we ignore content */
                 RESP_REDIR, /* responses for which we rewrite Location */
-                LOCATION;   /* the host we are redirected to */
+                LOCATION,   /* the host we are redirected to */
+                AUTHORIZATION;  /* the Authorisation header */
 
 extern char *e500,  /* default error 500 page contents */
             *e501,  /* default error 501 page contents */
-            *e503;  /* default error 503 page contents */
+            *e503,  /* default error 503 page contents */
+            *e414;  /* default error 414 page contents */
 
 /* Header types */
 #define HEADER_ILLEGAL              -1
@@ -415,13 +447,19 @@ extern void kill_be(struct sockaddr_in *);
 /*
  * Find if a host is in our list of back-ends
  */
-extern int is_be(char *, struct sockaddr_in *, char *);
+extern int is_be(char *, struct sockaddr_in *, char *, char *, GROUP *);
 
 /*
  * Add explicit port number (if required)
  */
 extern char *add_port(char *, struct sockaddr_in *);
 
+/*
+ * Non-blocking version of connect(2). Does the same as connect(2) but
+ * ensures it will time-out after a much shorter time period CONN_TO.
+ */
+extern int  connect_nb(int, struct sockaddr *, socklen_t);
+
 /*
  * Prune the expired sessions and dead hosts from the table;
  * runs every session_to seconds (if needed)
@@ -439,13 +477,24 @@ extern void *thr_resurect(void *);
  */
 extern void config_parse(int, char **);
 
+/*
+ * RSA ephemeral keys: how many and how often
+ */
+#define N_RSA_KEYS  11
+#define T_RSA_KEYS  300
+
 /*
  * return a pre-generated RSA key
  */
-extern RSA *RSA_tmp_callback(SSL *, int, int);
+extern RSA  *RSA_tmp_callback(SSL *, int, int);
+
+/*
+ * Pre-generate ephemeral RSA keys
+ */
+extern int  init_RSAgen(void);
 
 /*
  * Periodically regenerate ephemeral RSA keys
  * runs every T_RSA_KEYS seconds
  */
-void *thr_RSAgen(void *);
+extern void *thr_RSAgen(void *);
